import http.client as httplib
import os
import hmac
import json
import hashlib
import datetime
import urllib.parse as urlparse

import yaml

from .config import get_config
from .url import CxenseURL
from .decorators import with_env

class CX(object):

    @with_env('home')
    def __init__(self, env_fp=None, use='default'):
        if env_fp is not None:
            with open(env_fp, 'r') as stream:
                env = yaml.load(stream)
                self._config['username'] = env['cxense_api'][use]['username']
                self._config['secret'] = env['cxense_api'][use]['secret']
                self._config['apiserver'] = env['cxense_api'][use]['apiserver']
                self._config['siteId'] = env['cxense_api'][use]['site_id']

    def get_date(self, connection):
        try:
            connection.request("GET", "/public/date")
            return json.load(connection.getresponse())['date']
        except:
            pass

        return datetime.datetime.utcnow().isoformat() + "Z"

    def execute(self, path, content):
        if path.startswith('http'):
            url = urlparse.urlparse(path)
        else:
            url = urlparse.urlparse(urlparse.urljoin(self._config['apiserver'], path))

        connection = (httplib.HTTPConnection if url.scheme == 'http' else httplib.HTTPSConnection)(url.netloc)
        try:
            dt = self.get_date(connection)
            signature = hmac.new(self._config['secret'].encode('utf-8'), dt.encode('utf-8'), digestmod=hashlib.sha256).hexdigest()
            headers = {"X-cXense-Authentication": "username=%s date=%s hmac-sha256-hex=%s" % (self._config['username'], dt, signature)}
            headers["Content-Type"] = "application/json; charset=utf-8"
            connection.request("GET" if content is None else "POST", url.path + ("?" + url.query if url.query else ""), content, headers)
            response = connection.getresponse()
            status, header, content = response.status, response.getheader('Content-Type', ''), response.read()
            if status != 200:
                raise Exception(status, str(content.decode('utf-8')))

            return status, header, content
        finally:
            connection.close()
